import time

from spikewrap.pipeline.full_pipeline import run_full_pipeline


config_name = "test_default"


def run_full_sorting_pipeline_on_mouse(mtd, sorter="kilosort2_5"):
    base_path = mtd.root.parent
    sessions_and_runs = {
        s.session_path_derivatives.stem: [
            x.stem for x in list(s.session_path_raw.rglob("*run*"))
        ]
        for s in mtd.sessions
    }
    if len(sessions_and_runs.values()) == 0:
        print("no runs found")
        raise ValueError()
    t = time.time()

    run_full_pipeline(
        base_path,
        mtd.mouse_id,
        sessions_and_runs,
        config_name,
        sorter,
        concat_sessions_for_sorting=False,
        concat_runs_for_sorting=False,
        existing_preprocessed_data="skip_if_exists",  # this is kind of confusing...
        existing_sorting_output="skip_if_exists",
        overwrite_postprocessing=True,
        delete_intermediate_files=(),
        slurm_batch=True,
    )

    print(f"TOOK {time.time() - t}")



import pathlib


def get_output_dir(
    mouse_id,
    data_dir,
):
    data_dir = pathlib.Path(data_dir)
    return data_dir / mouse_id / "histology"


def voxel_sizes(recipe_path):
    import yaml

    with open(str(recipe_path), "r") as stream:
        try:
            params = yaml.safe_load(stream)
            return params["VoxelSize"]
        except yaml.YAMLError as exc:
            print(exc)


def get_brain_path(
    mouse_id,
    brain_dir,
    channel_id="2",
):
    path = (
        pathlib.Path(brain_dir) / mouse_id / "stitchedImages_100" / channel_id
    )
    return path


def brainreg_command(mouse_id, mouse_dir, root_dir, serial2p_dir, function="brainreg", atlas="allen_mouse_10um", additional=None):
    input_path = get_brain_path(mouse_id, serial2p_dir)
    output_path = get_output_dir(mouse_dir, data_dir=root_dir) / atlas
    print(input_path)
    print(output_path)
    recipe_path = list(input_path.parent.parent.glob("recipe*"))[0]
    voxels = voxel_sizes(recipe_path)
    additional = f"--additional {input_path.parent / additional}" if additional is not None else ""
    cmd = f"{function} {input_path} {output_path} {additional} -v {voxels['Z']} {voxels['X']} {voxels['Y']} --orientation psr --atlas {atlas}"
    return cmd


def brainreg_commandline(
    mouse_id, mouse_dir, root_dir, brain_dir, function="brainreg", atlas="kim_mouse_10um", logs_path=None, additional=None
):
    command = brainreg_command(mouse_id, mouse_dir, root_dir, brain_dir, function=function, atlas=atlas, additional=additional)
    # subprocess.run(command,
    #                executable="/bin/bash",
    #                shell=True,
    #                )



from pp_slurm import slurm_functions
from pathlib import Path

from breg_util.brainreg_command_line_util import brainreg_command


def brainreg_slurm(mtd, additional=None):
    # This is a hack right now and passes the actually used command via the additional_string
    # command, the rest is more or less irrelevant.
    kwarg_dict = {
        "slurm_batch": True,
        "logs_path": Path(f"{mtd.mouse_dir}") / "logs",
        "mouse_id": f"{mtd.mouse_id}",
        "mouse_dir": f"{mtd.mouse_dir}",
        "root_dir": f"{mtd.root}",
        "function": "brainreg",
        "atlas": "kim_mouse_10um",
        "module_string": "brainglobe",
        "additional_string": brainreg_command(mtd.mouse_id,
                                              mtd.mouse_dir,
                                              mtd.root,
                                              mtd.serial2p_dir,
                                              function="brainreg",
                                              atlas="kim_mouse_10um",
                                              additional=additional),
    }
    slurm_functions.run_brainreg_slurm(**kwarg_dict)
